"""The mono.surf package."""

from .client import MonosurfClient
from .schemas import (
    MerchantApiKeySchema,
    MerchantCreateSchema,
    MerchantSchema,
    MonobankAccountSchema,
    MonobankClientCreateSchema,
    MonobankClientSchema,
    MonobankJarSchema,
    PaycheckCreateResponseSchema,
    PaycheckCreateSchema,
)

__all__ = [
    "MonosurfClient",
    "MerchantSchema",
    "MerchantCreateSchema",
    "MonobankClientSchema",
    "MonobankClientCreateSchema",
    "MonobankAccountSchema",
    "MonobankJarSchema",
    "MerchantApiKeySchema",
    "PaycheckCreateSchema",
    "PaycheckCreateResponseSchema",
]



"""The schemas for the API."""

from typing import Annotated
from uuid import UUID

from pydantic import BaseModel, EmailStr, Field, field_validator
from pydantic_core import Url

from settings import settings


# region Merchant schemas
class BaseMerchantSchema(BaseModel):
    """The base schema for a Merchant."""

    first_name: str = Field(..., max_length=255)
    last_name: str = Field(..., max_length=255)

    username: str = Field(..., max_length=255)

    email: EmailStr = Field(..., max_length=255)


class MerchantCreateSchema(BaseMerchantSchema):
    """The schema for a Merchant."""


class MerchantSchema(BaseMerchantSchema):
    """The schema for a Merchant."""

    id: int


# endregion


# region Monobank client & accounts schemas
class MonobankClientBaseSchema(BaseModel):
    """The base schema for a Monobank client."""

    merchant_id: int


class MonobankClientCreateSchema(MonobankClientBaseSchema):
    """The schema for a Monobank client creation request."""

    token: str


class MonobankClientSchema(MonobankClientBaseSchema):
    """The schema for a Monobank client."""

    client_id: str
    name: str
    web_hook_url: str
    permissions: str


# endregion


# region Monobank Account & Jar schemas


class BaseMonobankAccountSchema(BaseModel):
    """The base schema for a Monobank account."""

    id: Annotated[str, Field(str, max_length=31)]
    send_id: Annotated[str, Field(str, max_length=14)] | None = None

    currency_code: int
    balance: int


class MonobankAccountSchema(BaseMonobankAccountSchema):
    """The schema for a Monobank account."""

    masked_pan: str | None = None

    type: str

    credit_limit: int

    cashback_type: str | None = None


class MonobankJarSchema(BaseMonobankAccountSchema):
    """The schema for a Monobank jar."""

    title: str
    description: str | None = None

    goal: int | None = None


# endregion


# region API key schemas
class MerchantApiKeyBaseSchema(BaseModel):
    """The base schema for a Merchant API key."""

    merchant_id: int


class MerchantApiKeyCreateSchema(MerchantApiKeyBaseSchema):
    """The schema for a Merchant API key creation request."""


class MerchantApiKeySchema(MerchantApiKeyBaseSchema):
    """The schema for a Merchant API key."""

    id: int

    key: UUID | None = Field(None, description="The API key. `null` if the key is masked.")
    masked_key: str = Field(
        ...,
        description="The masked API key. The first 4 and the last 4 characters are visible, the rest are masked.",
    )


# endregion


# region Paycheck schemas
class BasePaycheckSchema(BaseModel):
    """The base schema for a Paycheck."""

    for_customer_id: Annotated[int | str, Field(gt=0, description="The ID of the customer.")]

    @field_validator("for_customer_id")
    @classmethod
    def for_customer_id_must_be_int_or_str(cls, v: int | str) -> int | str:
        """Validate the `for_customer_id` field."""
        if not isinstance(v, (int, str)):
            raise ValueError("`for_customer_id` must be an integer or a string")
        if isinstance(v, str) and len(v) > 255:
            raise ValueError("`for_customer_id` must be at most 255 characters long")
        return v

    to_account_id: Annotated[str, Field(max_length=22, description="The ID of the account.")] | None = None
    to_jar_id: Annotated[str, Field(max_length=31, description="The ID of the jar.")] | None = None

    amount: Annotated[
        int,
        Field(
            gt=0,
            description="The amount of the paycheck in the smallest currency unit.",
        ),
    ]

    comment: str | None = None

    currency_symbol: str = settings.DEFAULT_CURRENCY_SYMBOL
    currency_code: int = settings.DEFAULT_CURRENCY_CODE


class PaycheckCreateSchema(BasePaycheckSchema):
    """The schema for a paycheck creation request."""


class PaycheckCreateResponseSchema(PaycheckCreateSchema):
    """The schema for a paycheck creation response."""

    id: UUID

    payment_link: Url

    is_paid: bool


# endregion



"""All the endpoints for the `monosurf` API service."""

from enum import Enum


class Endpoint(str, Enum):
    """The endpoints for the `monosurf` API service."""

    GET_ME = "/merchants/me"
    CREATE_MERCHANT = "/merchants/create"

    GET_MERCHANT_BY_ID = "/merchants/{merchant_id}"

    GET_MONOBANK_CLIENTS = "/monobank/clients"
    CREATE_MONOBANK_CLIENT = "/monobank/clients/create"

    GET_MONOBANK_ACCOUNTS = "/monobank/accounts"

    GET_MONOBANK_JARS = "/monobank/jars"

    GET_API_KEYS = "/api-keys"
    CREATE_API_KEY = "/api-keys/create"

    CREATE_PAYCHECK = "/paychecks/create"



"""Settings for the project."""

from pydantic_settings import BaseSettings, SettingsConfigDict


class Settings(BaseSettings):
    """Settings for the project."""

    model_config = SettingsConfigDict(env_file=".env", extra="ignore", env_prefix="MONOSURF__")

    API_KEY_HEADER_NAME: str = "X-API-Key"

    DEFAULT_CURRENCY_SYMBOL: str = "UAH"
    DEFAULT_CURRENCY_CODE: int = 980

    DEFAULT_COMMENT: str | None = None


settings = Settings()



"""The `MonosurfClient`."""

import httpx

from endpoints import Endpoint
from schemas import (
    MerchantApiKeySchema,
    MerchantCreateSchema,
    MerchantSchema,
    MonobankAccountSchema,
    MonobankClientCreateSchema,
    MonobankClientSchema,
    MonobankJarSchema,
    PaycheckCreateResponseSchema,
    PaycheckCreateSchema,
)
from settings import settings


class MonosurfClient:
    """The `MonosurfClient`."""

    def __init__(self, api_key: str):
        """Initialize the `MonosurfClient` with the given `api_key`."""
        self.api_host = "https://api.mono.surf"

        self._session = httpx.AsyncClient(headers={settings.API_KEY_HEADER_NAME: api_key})

    # region Syntax sugar
    async def _request(self, method: str, url: Endpoint, **kwargs) -> dict:
        """Make a request to the given `url` with the given `method`."""
        response = await self._session.request(method, f"{self.api_host}/{url.value.lstrip('/')}", **kwargs)
        response.raise_for_status()
        return response.json()

    async def _get(self, url: Endpoint, **kwargs) -> dict:
        """Make a GET request to the given `url`."""
        return await self._request("GET", url, **kwargs)

    async def _post(self, url: Endpoint, **kwargs) -> dict:
        """Make a POST request to the given `url`."""
        return await self._request("POST", url, **kwargs)

    # endregion

    async def get_me(self) -> MerchantSchema:
        """Get the current merchant."""
        return MerchantSchema(**await self._get(Endpoint.GET_ME))

    async def get_merchant_by_id(self, merchant_id: int) -> MerchantSchema:
        """Get the merchant by the given `merchant_id`. Available only for admins."""
        return MerchantSchema(**await self._get(Endpoint.GET_MERCHANT_BY_ID, params={"merchant_id": merchant_id}))

    async def create_merchant(self, merchant: MerchantCreateSchema) -> MerchantSchema:
        """Create a new merchant. Available only for admins."""
        return MerchantSchema(**await self._post(Endpoint.CREATE_MERCHANT, json=merchant.model_dump()))

    async def get_monobank_clients(self) -> list[MonobankClientSchema]:
        """Get all the Monobank clients."""
        return [MonobankClientSchema(**client) for client in await self._get(Endpoint.GET_MONOBANK_CLIENTS)]

    async def create_monobank_client(self, monobank_client: MonobankClientCreateSchema) -> MonobankClientSchema:
        """Create a new Monobank client."""
        return MonobankClientSchema(
            **await self._post(Endpoint.CREATE_MONOBANK_CLIENT, json=monobank_client.model_dump())
        )

    async def get_monobank_accounts(self) -> list[MonobankAccountSchema]:
        """Get all the Monobank accounts."""
        return [MonobankAccountSchema(**account) for account in await self._get(Endpoint.GET_MONOBANK_ACCOUNTS)]

    async def get_monobank_jars(self) -> list[MonobankJarSchema]:
        """Get all the Monobank jars."""
        return [MonobankJarSchema(**jar) for jar in await self._get(Endpoint.GET_MONOBANK_JARS)]

    async def create_api_key(self, merchant_id: int) -> dict:
        """Create a new API key. Available only for admins."""
        return await self._post(Endpoint.CREATE_API_KEY, json={"merchant_id": merchant_id})

    async def get_api_keys(self, merchant_id: int) -> list[MerchantApiKeySchema]:
        """Get all the API keys for the given `merchant_id`."""
        return [
            MerchantApiKeySchema(**api_key)
            for api_key in await self._get(Endpoint.GET_API_KEYS, params={"merchant_id": merchant_id})
        ]

    async def create_paycheck(self, paycheck: PaycheckCreateSchema) -> PaycheckCreateResponseSchema:
        """Create a new paycheck."""
        return PaycheckCreateResponseSchema(**await self._post(Endpoint.CREATE_PAYCHECK, json=paycheck.model_dump()))
